//
//  XYRTCEngine+AudioVideo.h
//  API
//
//  Created by 高超 on 2021/3/9.
//

#import "XYRTCEngine.h"

NS_ASSUME_NONNULL_BEGIN

@interface XYRTCEngine (AudioVideo)

/// 开启音频，调用此方法来创建音频单元
- (void)startAudio;

/// 结束音频，调用此方法来销毁音频单元
- (void)stopAudio;

/// 麦克风
/// @param mute 是否静音
- (void)setLocalMicMute:(BOOL)mute;

/// 麦克风
/// @param mute 是否静音
/// @param reason 静音原因
- (void)setLocalMicMute:(BOOL)mute reason:(XYMuteReason)reason;

/// 麦克风是否静音
- (BOOL)isLocalMicMuted;

/// 切换扬声器
/// @param on YES：扬声器  NO：听筒
- (void)setSpeakerphoneOn:(BOOL)on;

/// 外放，mute后扬声器和听筒无音频播放
/// @param mute 是否静音
- (void)setSpeakerMute:(BOOL)mute;

/// 外放是否静音
- (BOOL)isSpeakerMuted;

/// 视频
/// @param mute 是否关闭
- (void)setLocalVideoMute:(BOOL)mute;

/// 视频是否关闭
- (BOOL)isLocalVideoMuted;

/// 本地视频方向
/// @param orientation 方向值
- (void)setVideoOrientation:(XYVideoOrientation)orientation;

/// 切换摄像头
/// @param direction 摄像头方向
- (void)switchCamera:(AVCaptureDevicePosition)direction;

/// 摄像头方向
- (AVCaptureDevicePosition)cameraPosition;

/// 本地视频镜像
///
/// 只有在前置摄像头时设置生效，后置摄像头默认不镜像，前置摄像头默认镜像
///
/// @param flip YES 镜像  NO 不镜像
- (void)setLocalVideoFlip:(BOOL)flip;

/// 本地视频是否镜像
- (BOOL)isLocalVideoFlipping;

/// 开关手电筒，后置摄像头时生效
/// @param mode 手电筒状态
- (void)switchTorchMode:(AVCaptureTorchMode)mode;

/// 设置手电筒等级
/// @param level 手电筒等级 0~1
- (void)torchLevel:(float)level;

/// 手电筒开关状态
- (AVCaptureTorchMode)torchMode;

/// 旁路音频开关（入会之前设置）
/// @param enable 是否打开
- (void)setMicDataByAEReadyEnabled:(BOOL)enable;

/// 获取一帧视频数据
/// @param sourceId 远端视频流id，获取本地视频时sourceId传@"LocalPreviewID"
- (NSData *)getVideoDataBySourceId:(NSString *)sourceId;

/// 配置视频输出
/// @param videoDimension 分辨率
- (void)setVideoConfiguration:(XYRTCVideoDimension)videoDimension;

/// 获取指定视频流
/// @param streamID 视频流ID (ID为LOCAL时获取本地视频流)
/// @param completion 回调 (size 大小 stream 视频流)
- (void)getVideoStreamWithID:(NSString *)streamID
                  completion:(void (^)(int size, void *stream))completion;

/// 获取语音能量值（XYSettings设置 needAudioMeter = YES 时生效）
/// @param completion 回调 (peak 检测是否成功 meters 测量值)
- (void)getAudioPeakMetersCompletion:(void (^)(bool peak, NSArray *meters))completion;

@end

NS_ASSUME_NONNULL_END
