//
//  XYRTCEngine+Interaction.h
//  API
//
//  Created by 高超 on 2021/3/9.
//

#import "XYRTCEngine.h"

NS_ASSUME_NONNULL_BEGIN

@interface XYRTCEngine (Interaction)

/// 开启白板
- (void)startWhiteboard;

/// 关闭白板
- (void)stopWhiteboard;

///// 获取当前白板状态
- (XYWhiteboardState)currentWhiteboardState;

/// 发送白板数据
/// @param message 内容
/// @param type 动作
- (void)sendWhiteboardMsg:(id)message type:(XYWhiteboardOperation)type;

/// 会控地址
- (NSString *)getMeetingControlUrl;

/// 举手发言
- (void)handsUp;

/// 取消举手
- (void)handsDown;

/// 结束发言
- (void)endSpeech;

/// 签到
- (void)signIn:(XYInteractiveInfo *)interactiveInfo completion:(void (^)(BOOL, id _Nonnull))completion;

/// 开始分享content，分享屏幕仅支持iOS12及以上系统
/// @param type 类型
/// @param enableAnnotation 是否开启标注，只有type=XYContentType_Image时，此参数有效
/// @param bundleID extension bundle id
- (void)startContentSharing:(XYContentType)type
           enableAnnotation:(BOOL)enableAnnotation
                   bundleID:(NSString *_Nullable)bundleID;

/// 设置分享内容的方向
///
/// 会议中设置分享内容，默认值为 XYVideoOrientation_auto ，即跟随系统旋转
/// 此设置仅对单次会议生效，会议结束后会自动回复默认值
///
/// @param orientation 方向值
- (void)setContentOrientation:(XYVideoOrientation)orientation;

/// 发送content数据
/// @param content 数据内容
- (void)sendContentMsg:(XYRTCContent *)content;

/// 结束分享content
- (void)stopContentSharing;

/// 开启标注
- (void)startAnnotation;

/// 关闭标注
- (void)stopAnnotation;

/// 发送标注消息
/// @param message 消息数据（json）
- (void)sendAnnotationMessage:(NSString *)message;

@end

NS_ASSUME_NONNULL_END
