//
//  XYRTCEngine+Meeting.h
//  API
//
//  Created by 高超 on 2021/3/9.
//

#import "XYRTCEngine.h"

NS_ASSUME_NONNULL_BEGIN

@interface XYRTCEngine (Meeting)

/// 加入会议
/// 请使用onConnectionStateChanged：先查看是否连接服务器，再做呼叫操作，不要在onLoginResult：之后直接调用呼叫操作
/// @param number 会议号
/// @param password 密码
- (void)joinMeeting:(NSString *)number password:(NSString *)password;

/// 加入会议
/// 请使用onConnectionStateChanged：先查看是否连接服务器，再做呼叫操作，不要在onLoginResult：之后直接调用呼叫操作
/// @param number 会议号
/// @param password 密码
/// @param config 会议配置参数
- (void)joinMeeting:(NSString *)number
           password:(NSString *)password
             config:(XYRTCCallConfig * _Nullable)config;

/// 离开会议
- (void)leaveMeeting;

/// 离开会议
/// @param endRecord 是否结束录制，YES:结束  NO:不结束
- (void)leaveMeeting:(BOOL)endRecord;

/// 结束会议
- (void)endMeeting;

/// 是否在会议中
- (BOOL)isInMeeting;

/// 接受会议邀请
- (void)acceptInvitation;

/// 拒绝会议邀请
- (void)rejectInvitation;

/// 设置会议模式
/// @param mode 视频 or 语音
- (void)setMeetingMode:(XYMeetingMode)mode;

/// 请求参会者信息
/// @param startIndex 起始位置
/// @param length 长度 (单次请求最大20)
- (void)requestMembers:(NSInteger)startIndex length:(NSInteger)length;

/// 订阅参会者信息，订阅后，会下发指定的参会者，uris最多40个
/// @param uris callUrl
- (void)subscribeMembers:(NSArray<NSString *>*)uris;

/// 请求视频流
/// @param elements 需要请流的信息
- (void)requestLayout:(NSArray<XYRTCLayoutElement *>*)elements;

/// 创建视频
/// @param videoCanvas 画布类，绑定view和layout
- (void)setupVideo:(XYRTCVideoCanvas *)videoCanvas;

/// 更新视频
/// @param videoCanvas 画布类，绑定view和layout
- (void)updateVideo:(XYRTCVideoCanvas *)videoCanvas;

/// 上传截图
/// @param image 图片数据（最大不超过10M，jpg格式）
/// @param fileDisplayName 图片名称
/// @param completion 回调，success：YES成功 NO失败，responseObject：返回数据
- (void)uploadScreenShot:(UIImage *)image
         fileDisplayName:(NSString *)fileDisplayName
              completion:(void (^)(BOOL success, id responseObject))completion;

@end

NS_ASSUME_NONNULL_END
