//
//  XYRTCEngine+User.h
//  XYRTCEngine
//
//  Created by 高超 on 2022/4/13.
//

#import "XYRTCEngine.h"

NS_ASSUME_NONNULL_BEGIN

@interface XYRTCEngine (User)

/// 三方账号登录
/// @param userId 用户唯一id，最大50个字符，支持大小写英文字母、数字、上横线“-”、下横线“_”
/// @param username 用户名称
/// @param countryCode 国家码，不传默认+86
- (void)login:(NSString *)userId username:(NSString *)username countryCode:(NSString *_Nullable)countryCode;

/// 小鱼账号登录
/// @param account 账号
/// @param password 密码
- (void)loginXYLinkAccount:(NSString *)account password:(NSString *)password;

/// 三方授权登录
/// @param account 账户
/// @param token 认证token
- (void)loginByTokenWithAccount:(NSString *)account token:(NSString *)token;

/// 企业关联账号登录
/// @param account 账号
- (void)loginEnterpriseAccount:(NSString*)account;

/// 小鱼账号登录
/// @param account 账号
/// @param password 密码
/// @param countryCode 国家码，不传默认+86
- (void)loginXYAccount:(NSString *)account password:(NSString *)password countryCode:(NSString *_Nullable)countryCode;

/// 三方账号登录认证登录
/// @param extUserId 用户唯一id
/// @param authCode 认证授权码
/// @param displayName 用户名称
- (void)loginExtUserId:(NSString *)extUserId authCode:(NSString *)authCode displayName:(NSString *)displayName;

/// 三方授权登录
/// @param token 认证token
- (void)loginExtToken:(NSString *)token;

/// 单点登录
/// @param token 认证token
- (void)loginXYAccountWithToken:(NSString *)token;

/// 刷新access_token
/// @param refreshToken refreshToken，从登录成功回调中返回
/// @param completion 回调
- (void)refreshToken:(NSString *)refreshToken
          completion:(void(^)(XYUserInfo *userInfo))completion;

/// 登出
- (void)logout;

/// 是否已登录
- (BOOL)isLogined;

/// 修改名称
/// @param displayName 名字，用于会中显示
/// @param completion 回调
- (void)modifyDisplayName:(NSString *)displayName
               completion:(void (^)(BOOL success, id responseObject))completion;

@end

NS_ASSUME_NONNULL_END
