//
//  XYRTCBaseMeetingModels.h
//  xylink_app
//
//  Created by gao chenyang on 2020/12/9.
//  Copyright © 2020 ainemo. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import <ReplayKit/ReplayKit.h>
#import "XYRTCEngineEnums.h"

NS_ASSUME_NONNULL_BEGIN

/// 初始化参数
@interface XYRTCSettings : NSObject
/// 企业ID，在管理平台生成，不能为空
@property (nonatomic, copy) NSString *enterpriseID;
/// 客户端id，在管理平台生成
@property (nonatomic, copy) NSString *clientID;
/// 客户端秘钥，在管理平台生成
@property (nonatomic, copy) NSString *clientSecret;
/// server地址，默认小鱼云环境
@property (nonatomic, copy) NSString *server;
/// http端口号，默认80
@property (nonatomic, copy) NSString *httpPort;
/// https端口号，默认443
@property (nonatomic, copy) NSString *httpsPort;
/// log等级
@property (nonatomic, assign) XYRTCLogLevel logLevel;
/// log目录地址，默认：Documents/
@property (nonatomic, copy) NSString *logFilePath;
/// log文件名称，默认：vulture
@property (nonatomic, copy) NSString *logFileName;
/// 视频输出分辨率，默认720p
@property (nonatomic, assign) XYRTCVideoDimension videoDimension;
/// 摄像头方向，0:后置摄像头,1:前置摄像头，默认值：1
@property (nonatomic, assign) NSInteger defaultCameraId;
/// 是否打印log
@property (nonatomic, assign) BOOL enableLog;
/// 同传字幕开关
@property (nonatomic, assign) BOOL enableAiCaption;
/// 是否需要音量探测
@property (nonatomic, assign) BOOL needAudioMeter;
/// 是否需要上报当前发言者（最多3路）
@property (nonatomic, assign) BOOL needSpeakers;
/// 信任服务端证书，生产环境设置NO，测试环境设置YES，默认值：NO
@property (nonatomic, assign) BOOL trustCertificatesChain;
/// 安全的请求方法，默认值：NO
@property (nonatomic, assign) BOOL safeMethodRequest;
/// 获取XYRTCSettings单例
+ (XYRTCSettings *)settings;

@end


/// 用户信息
@interface XYUserInfo : NSObject
/// 用户id
@property (nonatomic, assign) long userId;
/// 设备ID
@property (nonatomic, assign) long deviceId;
/// 设备类型
@property (nonatomic, assign) int deviceType;
/// 用户名
@property (nonatomic, copy) NSString *displayName;
/// 号码
@property (nonatomic, copy) NSString *callNumber;
/// 鉴权token
@property (nonatomic, copy) NSString *accessToken;
/// 刷新token
@property (nonatomic, copy) NSString *refreshToken;
/// accessToken的有效时长
@property (nonatomic, assign) long validDuration;

@end


/// 呼叫配置
@interface XYRTCCallConfig : NSObject
/// 关闭音频入会
@property (nonatomic, assign) BOOL audioMute;
/// 关闭视频入会
@property (nonatomic, assign) BOOL videoMute;
/// 监会者
@property (nonatomic, assign) BOOL isMonitor;
/// 预约会议ID
@property (nonatomic, copy) NSString *scheduledEventId;

@end


/// 会议状态信息
@interface XYRTCMeetingStateInfo : NSObject
/// 呼叫标记
@property (nonatomic, assign) NSInteger callIndex;
/// 会议状态
@property (nonatomic, assign) XYMeetingState state;
/// 呼叫类型
@property (nonatomic, assign) XYCallType callType;
/// 会议号或终端号
@property (nonatomic, copy) NSString *meetingNumber;
/// 会议id
@property (nonatomic, copy) NSString *meetingId;
/// 会议室名称或终端名称
@property (nonatomic, copy) NSString *meetingName;
/// 原因，成功时返回STATE:200，失败时返回对应错误码
@property (nonatomic, copy) NSString *reason;
/// 等候室开关
@property (nonatomic, assign) BOOL enableWaitingRoom;
/// 等候室状态
@property (nonatomic, assign) XYWaitingRoomState waitingRoomState;

@end


/// 来电信息
@interface XYInComingCallInfo : NSObject
/// 呼叫标记
@property (nonatomic, assign) NSInteger callIndex;
/// 状态
@property (nonatomic, assign) XYMeetingState state;
/// 呼叫类型
@property (nonatomic, assign) XYCallType callType;
/// 设备类型
@property (nonatomic, assign) XYDeviceType remoteType;
/// 呼叫模式
@property (nonatomic, assign) XYMeetingMode callModel;
/// 名称
@property (nonatomic, copy) NSString *remoteName;
/// 呼叫标识
@property (nonatomic, copy) NSString *remoteUri;
/// 呼叫号码
@property (nonatomic, copy) NSString *callerNumber;
/// 需要展示的号码
@property (nonatomic, copy) NSString *displayNumber;
/// 群组呼叫
@property (nonatomic, copy) NSString *groupVc;
/// 呼叫断开的原因，state = XYMeetingState_DisConnected 时有值
@property (nonatomic, copy) NSString *reason;

@end


/// 视频流信息
@interface XYRTCLayoutElement : NSObject
/// 参会人ID
@property (nonatomic, assign) int participantId;
/// 分辨率
@property (nonatomic, assign) XYCustomVideoResolution resolution;
/// 布局优先级
@property (nonatomic, assign) XYCustomLayoutPriority priority;
/// 视频质量
@property (nonatomic, assign) XYCustomVideoQuality quality;
/// 是否请求人脸信息
@property (nonatomic, assign) BOOL reqFaceDetect;

@end


/// 分享
@interface XYRTCContent : NSObject
/// 分享类型
@property (nonatomic, assign) XYContentType type;
/// 图片数据
@property (nonatomic, strong) UIImage *image;
/// 图片帧率，默认值：5
@property (nonatomic, assign) NSInteger imageFrameRate;
/// 采集到的数据
@property (nonatomic, assign) CMSampleBufferRef buffer;
/// 采集到的数据类型
@property (nonatomic, assign) RPSampleBufferType bufferType API_AVAILABLE(ios(10.0));

@end


/// 参会者的信息
@interface XYMemberInfo : NSObject
/// 参会人ID
@property (nonatomic, assign) int participantId;
/// fecc能力值
@property (nonatomic, assign) int feccOri;
/// weight
@property (nonatomic, assign) int  weight;
/// 设备类型
@property (nonatomic, assign) XYDeviceType deviceType;
/// 设备ID
@property (nonatomic, copy) NSString *deviceId;
/// 设备名
@property (nonatomic, copy) NSString *deviceName;
/// 设备别名
@property (nonatomic, copy) NSString *deviceAlias;
/// 用户id（三方登录传入的id）
@property (nonatomic, copy) NSString *extUserId;
/// 是否是电话
@property (nonatomic, assign) BOOL isTelephone;
/// 是否观察模式
@property (nonatomic, assign) BOOL isObserver;
/// 是否只有语音
@property (nonatomic, assign) BOOL isAudioOnly;
/// 是否静音
@property (nonatomic, assign) BOOL isAudioMute;
/// 是否关闭视频
@property (nonatomic, assign) BOOL isVideoMute;
/// 是否被force到大屏
@property (nonatomic, assign) BOOL isForceFullScreen;
/// requestRoster主动请求
@property (nonatomic, assign) BOOL isRequested;
/// 是否允许标注
@property (nonatomic, assign) BOOL isAnnotationEnable;

@end


/// 参会者信息
@interface XYMeetingInfo : NSObject
/// people信息，目前最大接受20个参会信息
@property (nonatomic, strong) NSArray <XYMemberInfo *> *peopleRosters;
/// content信息，目前最大接受20个参会信息
@property (nonatomic, strong) NSArray <XYMemberInfo *> *contentRosters;
/// 远端人数，会议中除自己外的总参会人数
@property (nonatomic, assign) int numberOfPeople;
/// 通话模式
@property (nonatomic, assign) XYMeetingMode callMode;
/// content发送者id
@property (nonatomic, assign) int contentSenderPid;
/// 正在讲话的人id
@property (nonatomic, assign) int activeSpeakerPid;
/// 订阅序列
@property (nonatomic, assign) int subscribeSequence;
/// 是否接收分享视频
@property (nonatomic, assign) BOOL recvingVideoContent;
/// 是否接收分享图片
@property (nonatomic, assign) BOOL recvingPictureContent;
/// 其他参会者
@property (nonatomic, assign) BOOL hasOtherParticipants;

@end


/// 全量参会者信息
@interface XYBulkMemberInfo : NSObject
/// 参会者id
@property (nonatomic, assign) NSInteger participantId;
/// 设备类型
@property (nonatomic, assign) XYDeviceType deviceType;
/// 呼叫标识
@property (nonatomic, strong) NSString *callUri;
/// 名称
@property (nonatomic, strong) NSString *displayName;
/// 别名
@property (nonatomic, strong) NSString *alias;
/// 接收发送全部关闭
@property (nonatomic, assign) BOOL rxtxMuteAll;
/// 音频关闭
@property (nonatomic, assign) BOOL isAudioMute;
/// 视频关闭
@property (nonatomic, assign) BOOL isVideoMute;
/// 等候室
@property (nonatomic, assign) BOOL onHold;

@end


/// 全量参会者信息
@interface XYBulkMeetingInfo : NSObject
/// 分组id
@property (nonatomic, assign) NSInteger sessionId;
/// 参会者总数
@property (nonatomic, assign) NSInteger totalNum;
/// roster类型
@property (nonatomic, assign) XYBulkRosterType rosterType;
/// 增加的人
@property (nonatomic, strong) NSArray<XYBulkMemberInfo *> *addParticipants;
/// 变化的人
@property (nonatomic, strong) NSArray<XYBulkMemberInfo *> *changeParticipants;
/// 减少的人
@property (nonatomic, strong) NSArray<NSString *> *delParticipants;

@end


/// 视频流信息
@interface XYStreamInfo : NSObject
/// 远端名称
@property (nonatomic, copy) NSString *remoteName;
/// 视频流ID
@property (nonatomic, copy) NSString *streamID;
/// 参会人ID
@property (nonatomic, copy) NSString *participantId;
/// 是否是content
@property (nonatomic, assign) BOOL isContent;
/// 是否静音
@property (nonatomic, assign) BOOL isAudioMute;
/// 是否关闭视频
@property (nonatomic, assign) BOOL isVideoMute;
/// 是否在讲话
@property (nonatomic, assign) BOOL isActiveSpeaker;
/// 视频宽度
@property (nonatomic, assign) NSInteger videoWidth;
/// 视频高度
@property (nonatomic, assign) NSInteger videoHeight;
/// 视频流状态
@property (nonatomic, assign) XYVideoStreamState streamState;
/// 远端类型
@property (nonatomic, assign) XYDeviceType deviceType;
/// 远端ID
@property (nonatomic, assign) long remoteID;
/// 远端uri
@property (nonatomic, copy) NSString *remoteUri;
/// 视频关闭原因
@property (nonatomic, assign) XYMuteReason videoMuteReason;
/// feccOri
@property (nonatomic, assign) unsigned int feccOri;
/// 远端别名
@property (nonatomic, copy) NSString *remoteAlias;
/// 用户id（三方登录传入的id）
@property (nonatomic, copy) NSString *extUserId;

@end


/// 单个视频流分辨率信息
@interface XYVideoStreamInfo : NSObject
/// 参会者id
@property (nonatomic, assign) NSUInteger participantId;
/// 视频流宽度
@property (nonatomic, assign) NSUInteger width;
/// 视频流高度
@property (nonatomic, assign) NSUInteger height;
/// 视频流原始宽度
@property (nonatomic, assign) NSUInteger originalWidth;
/// 视频流原始高度
@property (nonatomic, assign) NSUInteger originalHeight;

@end


/// 会控信息
@interface XYMeetingControlInfo : NSObject
/// 麦克风状态
@property (nonatomic, assign) XYMeetingCtrlMicState state;
/// 主会场模式（判断主会场模式需要结合主会场ID）
@property (nonatomic, assign) BOOL isChairmanMode;
/// 麦克风是否禁用
@property (nonatomic, assign) BOOL muteIsDisabled;
/// fecc是否禁用
@property (nonatomic, assign) BOOL feccIsDisabled;
/// 共享内容是否禁用
@property (nonatomic, assign) BOOL contentIsDisabled;
/// 录制是否禁用
@property (nonatomic, assign) BOOL recordIsDisabled;
/// 白板是否禁用
@property (nonatomic, assign) BOOL whiteboardIsDisabled;
/// 标注是否禁用
@property (nonatomic, assign) BOOL annotationIsDisabled;
/// 主会场ID
@property (nonatomic, copy) NSString *chairmanUri;
/// 是否禁止外放（禁止收听）
@property (nonatomic, copy) NSString *isMuteSpeaker;
/// 修改名称
@property (nonatomic, copy) NSString *reName;

@end


/// 会控字幕
@interface XYCaptionInfo : NSObject
/// 展示字幕
@property (nonatomic, assign) BOOL display;
/// 字幕内容
@property (nonatomic, copy) NSString * content;
/// 位置
@property (nonatomic, assign) XYCaptionLocation location;
/// 是否滚动
@property (nonatomic, assign) BOOL scroll;
/// 速度
@property (nonatomic, copy) NSString *scrollSpeed;
/// 背景色
@property (nonatomic, copy) NSString * bgColor;
/// 背景透明度
@property (nonatomic, assign) CGFloat bgColorAlpha;
/// 字体颜色
@property (nonatomic, copy) NSString * fontColor;
/// 字体大小
@property (nonatomic, assign) XYCaptionFontSize fontType;
/// notice
@property (nonatomic, copy) NSString *notice;
/// fontSize
@property (nonatomic, assign) CGFloat fontSize;
/// 持续时间
@property (nonatomic, assign) int  duration;

@end


/// 会控互动
@interface XYInteractiveInfo : NSObject
/// 会议名称
@property (nonatomic, copy) NSString *conferenceName;
/// 会议号
@property (nonatomic, copy) NSString *conferenceNo;
/// 会议id
@property (nonatomic, copy) NSString *meetingId;
/// 交互id
@property (nonatomic, copy) NSString *questionnaireId;
/// 请求地址
@property (nonatomic, copy) NSString *url;
/// 交互类型
@property (nonatomic, assign) NSInteger voteType;
/// 是否支持人脸签到
@property (nonatomic, assign) BOOL supportFace;
/// 是否支持扫码签到
@property (nonatomic, assign) BOOL supportScan;
/// 是否自动结束
@property (nonatomic, assign) BOOL endAuto;
/// 自动结束时间
@property (nonatomic, assign) long endTime;
/// 主持人手动结束时间
@property (nonatomic, assign) long endUtcTime;
/// 是否自动提交
@property (nonatomic, assign) BOOL supportAutoSubmit;

@end


/// 同传字幕
@interface XYAiCaptionInfo : NSObject
/// 参会人ID
@property (nonatomic, assign) NSInteger pid;
/// 源
@property (nonatomic, copy) NSString *src;
/// 翻译后
@property (nonatomic, copy) NSString *target;
/// 源语言类型 zh/en
@property (nonatomic, copy) NSString *srcLang;
/// 目标语言类型 zh/en
@property (nonatomic, copy) NSString *targetLang;
/// 一句话是否已结束
@property (nonatomic, assign) BOOL isEnd;
/// 收三路，同时只有一路active
@property (nonatomic, assign) BOOL isActive;
/// 用户callurl
@property (nonatomic, copy) NSString *url;
/// 用户 displayname
@property (nonatomic, copy) NSString *dn;

@end


/// 视频渲染
@interface XYRTCVideoCanvas : NSObject
/// 承载视频的父view
@property (nonatomic, strong) UIView *view;
/// 视频流信息，isLocal=YES时可不传或传nil
@property (nonatomic, strong) XYStreamInfo *stream;
/// 是否是本地视频，默认NO
@property (nonatomic, assign) BOOL isLocal;
/// 使用OpenGL渲染，默认NO
@property (nonatomic, assign) BOOL useOpenGL;
/// 清空
- (void)clear;

@end


/// 发言者
@interface XYRTCSpeakers : NSObject
/// 发言者语音能量值
@property (nonatomic, assign) NSInteger energyAverage;
/// 发言者calluri
@property (nonatomic, copy) NSString *calluri;
/// 发言者参会id
@property (nonatomic, assign) NSInteger participantId;
/// 发言者名称
@property (nonatomic, copy) NSString *displayName;
/// 发言者用户id
@property (nonatomic, copy) NSString *extUserId;

@end


/// 录制信息
@interface XYRecordingInfo : NSObject
/// 录制状态
@property (nonatomic, assign) XYRecordingState state;
/// 录制发起者
@property (nonatomic, assign) BOOL isRecordingOwer;
/// 远端名称
@property (nonatomic, copy) NSString *displayName;
/// 呼叫标识
@property (nonatomic, copy) NSString *callUri;
/// 录制异常原因
@property (nonatomic, copy) NSString *reason;

@end


/// 线路信息
@interface XYSitePathInfo : NSObject
/// 线路名称
@property (nonatomic, copy) NSString *displayName;
/// 线路标识
@property (nonatomic, copy) NSString *sitePathId;
/// 线路类型 0 公有 1 私有
@property (nonatomic, assign) NSInteger sitePathType;
/// 网络类型 0 内网 1 外网
@property (nonatomic, assign) NSInteger networkType;

@end


/// 线路选择
@interface XYSitePath : NSObject
/// 已选择的线路
@property (nonatomic, copy) NSString *currentSitePathId;
/// 所有线路列表
@property (nonatomic, strong) NSArray <XYSitePathInfo *>*sitePathList;

@end


/// 出入会通知
@interface XYInOutNotice : NSObject
/// 用户id（三方登录传入的id）
@property (nonatomic, copy) NSString *extUserlId;
/// 名称
@property (nonatomic, copy) NSString *displayName;
/// yes 入会 no 退会
@property (nonatomic, assign) BOOL isIn;

@end


/// 网络测试
@interface XYNetTestInfo : NSObject
/// 测试带宽
@property (nonatomic, assign) int setBandwidth;
/// 实际带宽
@property (nonatomic, assign) int bandwidth;
/// 网络丢包
@property (nonatomic, assign) int lostRate;
/// 抖动
@property (nonatomic, assign) int jitter;
/// 往返时间
@property (nonatomic, assign) int rtt;
/// 乱序
@property (nonatomic, assign) int missorder;
/// 数据包数量
@property (nonatomic, assign) int pktNum;

@end


/// 网络测试结果
@interface XYNetTestResult : NSObject
/// 发送质量 1：非常好  2：良好  3：一般  4：较差  5：很差
@property (nonatomic, assign) int sendQualityLevel;
/// 接收质量 1：非常好  2：良好  3：一般  4：较差  5：很差
@property (nonatomic, assign) int recvQualityLevel;
/// 发送详情
@property (nonatomic, strong) NSArray <XYNetTestInfo *> *sendDetail;
/// 接收详情
@property (nonatomic, strong) NSArray <XYNetTestInfo *> *recvDetail;

@end


/// 人脸位置信息
@interface XYRTCFacePosition : NSObject
/// 人脸id
@property (nonatomic, assign) long faceId;
/// 左边距
@property (nonatomic, assign) long left;
/// 上边距
@property (nonatomic, assign) long top;
/// 右边距
@property (nonatomic, assign) long right;
/// 下边距
@property (nonatomic, assign) long bottom;

@end


/// 人脸信息
@interface XYRTCFace : NSObject
/// 参会者id
@property (nonatomic, assign) long participantId;
/// 类型，0：人脸识别 2：电子铭牌
@property (nonatomic, assign) int type;
/// 序列号
@property (nonatomic, assign) int seq;
/// 呼叫标识
@property (nonatomic, copy) NSString *callUri;
/// 位置信息
@property (nonatomic, strong) NSMutableArray <XYRTCFacePosition *>*positionVec;

@end

NS_ASSUME_NONNULL_END
