//
//  XYRTCEngineDelegate.h
//  xylink_app
//
//  Created by 高超 on 2020/9/21.
//  Copyright © 2020 ainemo. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "XYRTCEngineDefines.h"

@class XYUserInfo;
@class XYRTCMeetingStateInfo;
@class XYMeetingInfo;
@class XYStreamInfo;
@class XYMeetingControlInfo;
@class XYCaptionInfo;
@class XYRecordingInfo;
@class XYRTCSpeakers;
@class XYInOutNotice;

@protocol XYRTCEngineDelegate <NSObject>
@optional
/// 登录结果
/// @param userInfo 用户信息，登录失败时返回nil
/// @param errorCode 错误码，登录成功时返回nil
- (void)onLoginResult:(XYUserInfo *)userInfo errorCode:(NSString *)errorCode;

/// 登出
- (void)onLogout;

/// 账号在其他设备登录，被强制下线
- (void)onAccountWasKickOut;

/// 会议状态发生变化
/// @param info 会议状态信息
- (void)onMeetingStateChanged:(XYRTCMeetingStateInfo *)info;

/// 参会者信息发生变化
/// @param meetingInfo 参会者信息
- (void)onMeetingMemberChanged:(XYMeetingInfo *)meetingInfo;

/// 参会者信息发生变化(全量接口)
/// @param meetingInfo 参会者信息
- (void)onMeetingBulkMemberChanged:(XYBulkMeetingInfo *)meetingInfo;

/// 视频流信息发送变化
/// @param streamInfo 视频流信息
- (void)onMeetingStreamChanged:(NSArray<XYStreamInfo *> *)streamInfo;

/// 单个视频流分辨率信息
/// @param videoInfo 分辨率信息
- (void)onVideoStreamInfo:(XYVideoStreamInfo *)videoInfo;

/// 出入会通知
/// @param noticeInfos 参会者信息
- (void)onInOutNotice:(NSArray<XYInOutNotice *> *)noticeInfos;

/// 录制状态发生变化
/// @param recordingInfo 录制信息
- (void)onRecordingStateChanged:(XYRecordingInfo *)recordingInfo;

/// 分享状态发生变化
/// @param type 分享类型
/// @param state 状态
- (void)onContentShareStateChanged:(XYContentType)type state:(XYContentState)state;

/// 白板状态发生变化
/// @param state 状态
- (void)onWhiteboardStateChanged:(XYWhiteboardState)state;

/// 接收到白板消息
/// @param message 消息内容
/// @param operation 动作
- (void)onWhiteboardMsgReceived:(id)message operation:(XYWhiteboardOperation)operation;

/// 标注状态变化
/// @param state 状态
- (void)onAnnotationStateChanged:(XYAnnotationStatus)state;

/// 接收到标注消息
/// @param message 消息内容
/// @param operation 动作
- (void)onAnnotationMsgReceived:(id)message operation:(XYAnnotationOperation)operation;

/// 会控消息回调
/// @param info 消息内容
- (void)onMeetingControlChanged:(XYMeetingControlInfo *)info;

/// 接收到会议邀请或点对点呼叫
/// @param inComingCallInfo 远端信息
- (void)onMeetingInvited:(XYInComingCallInfo *)inComingCallInfo;

/// 连接状态发生变化
/// @param status 连接状态
- (void)onConnectionStateChanged:(XYWebSocketStatus)status;

/// 视频状态发生变化
/// @param state 状态
- (void)onVideoStatusChanged:(XYVideoState)state;

/// 网络等级发生变化
/// @param level 等级(共分为4个等级-> 1 2 3 4，由低到高分别为差，中，良，优)
- (void)onNetworkIndicatorChanged:(NSInteger)level;

/// 呼叫中心提示
/// @param msg 信息
- (void)onIMNotification:(NSString *)msg;

/// 远端人脸识别
/// @param face 人脸数据
- (void)onAiFace:(XYRTCFace *)face;

/// 本地人脸识别
/// @param face 人脸数据
- (void)onLocalFaceDetected:(XYRTCFace *)face;

/// 音频旁路回调
///
/// 返回本地音频原始数据，小端存储，单声道16比特PCM，采样率16kHz
///
/// @param size 数据大小
/// @param data 音频数据
- (void)onMicDataByAEReady:(NSInteger)size data:(void *)data;

/// 接收到字幕回调
/// @param info 字幕信息
- (void)onCaptionNotification:(XYCaptionInfo *)info;

/// 会控交互回调
/// @param state 交互状态
/// @param interactiveInfo 交互内容
- (void)onInteractiveEvent:(XYInteractiveState)state interactiveInfo:(XYInteractiveInfo *)interactiveInfo;

/// 会议锁定
/// @param locked 是否锁定，yes代表锁定，no代表解锁
- (void)onMeetingLocked:(BOOL)locked;

/// 主持人状态改变
/// @param meetingId 会议id
/// @param host YES：是主持人， NO：不是主持人
- (void)onMeetingHostChanged:(NSString *)meetingId host:(BOOL)host;

/// 联席主持人状态改变
/// @param meetingId 会议id
/// @param coHost YES：是联席主持人， NO：不是联席主持人
- (void)onMeetingCoHostChanged:(NSString *)meetingId coHost:(BOOL)coHost;

/// 会议室owner（会议室所有者）状态回调
/// @param meetingId 会议id
/// @param owner YES：是所有者  NO：不是所有者
- (void)onMeetingOwnerChanged:(NSString *)meetingId owner:(BOOL)owner;

/// 会控请求操作您的摄像头或麦克风
/// @param meetingId 会议id
/// @param query 请求类型
- (void)onMeetingMuteQuery:(NSString *)meetingId query:(XYMediaQuery)query;

/// 会控清除举手发言
///
/// 会议中申请举手发言之后，会控清除举手时触发此回调
///
- (void)onMeetingForceHandDown;

/// 同传字幕回调
/// @param info 详细信息
- (void)onAiCaptionInfo:(XYAiCaptionInfo *)info;

/// 音频单元初始化失败
/// @param error 错误信息
- (void)onAudioInitFailed:(NSString *)error;

/// 音频被中断
/// @param began 开始/结束
- (void)onAudioInterrupt:(BOOL)began;

/// 发言者
/// @param speakers 最多三路发言者
- (void)onSpeakersChanged:(NSArray <XYRTCSpeakers *>*)speakers;

/// 啸叫检测
/// @param detected 是否探测到啸叫
- (void)onHowlingDetected:(BOOL)detected;

/// 系统音量变化
/// @param value 音量值
- (void)onSystemVolumeChanged:(CGFloat)value;

/// 本地麦克风音量
/// @param value 音量值
- (void)onLocalMicrophoneDecibelChanged:(CGFloat)value;

/// 代理服务器测试接口回调
/// @param result 结果 成功返回OK
- (void)onSocksProxyValidateResult:(NSString *)result;

/// XYRTCSDK 错误回调，详见错误码
/// @param code 错误码
/// @param message 错误信息
- (void)onError:(NSString *)code message:(NSString *)message;
@end
