//
//  XYRTCEngineEnums.h
//  XYlink
//
//  Created by yangxudong on 2020/12/16.
//  Copyright © 2020 ainemo. All rights reserved.
//

#ifndef XYRTCEngineEnums_h
#define XYRTCEngineEnums_h

/// log等级
typedef NS_ENUM(NSUInteger, XYRTCLogLevel) {
    /// 错误信息
    XYRTCLogLevel_Error,
    /// 警告信息
    XYRTCLogLevel_Warn,
    /// 媒体信息
    XYRTCLogLevel_Info,
    /// 调试信息
    XYRTCLogLevel_Debug,
};

/// 设置视频输出分辨率
typedef NS_ENUM(NSUInteger, XYRTCVideoDimension) {
    /// 320 x 180
    XYRTCVideoDimension_180P,
    /// 640 x 360
    XYRTCVideoDimension_360P,
    /// 1280 x 720
    XYRTCVideoDimension_720P,
};

/// 设备类型
typedef NS_ENUM(NSUInteger, XYDeviceType)  {
    /// 未知
    XYDeviceType_Unknown,
    /// 微信小程序
    XYDeviceType_WeChat,
    /// 移动端
    XYDeviceType_Contact = 1,
    /// 硬件
    XYDeviceType_Device,
    /// 会议室
    XYDeviceType_MeetingRoom,
    /// PSTN
    XYDeviceType_PSTN,
    /// group
    XYDeviceType_Group,
    /// 桌面端
    XYDeviceType_Desktop = 5,
    /// H323
    XYDeviceType_H323,
    /// 大鱼
    XYDeviceType_Bruce,
    /// TVBox
    XYDeviceType_TVBox,
    /// Shuttle
    XYDeviceType_Shuttle,
    /// PSTNGW
    XYDeviceType_PSTNGW,
    /// WebRTC
    XYDeviceType_WebRTC,
};

/// 接收分辨率
typedef NS_ENUM(NSUInteger, XYCustomVideoResolution) {
    /// 320 x 180
    XYCustomVideoResolution_180P      = 100,
    /// 640 x 360
    XYCustomVideoResolution_360P      = 200,
    /// 1280 x 720
    XYCustomVideoResolution_720P      = 300,
    /// 1080 x 1920
    XYCustomVideoResolution_1080P     = 400,
};

/// 布局优先级
typedef NS_ENUM(NSUInteger, XYCustomLayoutPriority) {
    /// 高
    XYCustomLayoutPriority_High     = 100,
    /// 中
    XYCustomLayoutPriority_Normal   = 200,
    /// 低
    XYCustomLayoutPriority_Low      = 300,
};

/// 视频质量
typedef NS_ENUM(NSUInteger, XYCustomVideoQuality) {
    /// 高
    XYCustomVideoQuality_High     = 100,
    /// 中
    XYCustomVideoQuality_Normal   = 200,
    /// 低
    XYCustomVideoQuality_Base     = 300,
};

/// 会议模式
typedef NS_ENUM(NSUInteger, XYMeetingMode) {
    /// 音视频
    XYMeetingMode_AudioVideo,
    /// 音频
    XYMeetingMode_AudioOnly,
};

/// 会控请求
typedef NS_ENUM(NSUInteger, XYMediaQuery) {
    /// 关闭麦克风
    XYMediaQuery_MuteMicrophone,
    /// 关闭摄像头
    XYMediaQuery_MuteCamera,
};

/// 呼叫的终端类型
typedef NS_ENUM(NSUInteger, XYCallType) {
    /// 点对点
    XYCallType_PIP,
    /// 会议室
    XYCallType_Conference,
    /// 硬件
    XYCallType_Device,
    /// 云呼叫中心
    XYCallType_CCC,
    /// 直播
    XYCallType_LiveRoom,
};

/// 会议状态
typedef NS_ENUM(NSUInteger, XYMeetingState) {
    /// 正在连接
    XYMeetingState_Connecting,
    /// 已连接
    XYMeetingState_Connected,
    /// 断开连接
    XYMeetingState_DisConnected,
};

/// 等候室状态
typedef NS_ENUM(NSUInteger, XYWaitingRoomState) {
    /// 未知
    XYWaitingRoomState_Unknown,
    /// 等待主持人批准
    XYWaitingRoomState_WaitApprove,
    /// 等待主持人入会
    XYWaitingRoomState_WaitHostJoin,
};

/// websocket连接状态
typedef NS_ENUM(NSUInteger, XYWebSocketStatus) {
    /// 断开连接
    XYWebSocketStatus_Disconnected,
    /// 已连接
    XYWebSocketStatus_Connected,
    /// 正在连接
    XYWebSocketStatus_Connecting,
};

/// 本地视频方向
typedef NS_ENUM(NSUInteger, XYVideoOrientation) {
    /// 跟随系统
    XYVideoOrientation_Auto,
    /// 竖屏
    XYVideoOrientation_Portrait,
    /// 横屏 home键朝左
    XYVideoOrientation_LandscapeLeft,
    /// 横屏 home键朝右
    XYVideoOrientation_LandscapeRight,
};

/// 挂断方式
typedef NS_ENUM(NSUInteger, XYHangUpMode) {
    /// 默认挂断
    XYHangUpMode_Default,
    /// 会中邀请挂断
    XYHangUpMode_Invite,
};

/// 全量roster类型
typedef NS_ENUM(NSUInteger, XYBulkRosterType) {
    /// 全量数据
    XYBulkRosterType_Full,
    /// 增量数据
    XYBulkRosterType_Increment,
};

/// 视频状态
typedef NS_ENUM(NSUInteger, XYVideoState) {
    /// 正常状态，ui不需要显示
    XYVideoState_Normal,
    /// 本地网络不稳定
    XYVideoState_LowAsLocalBW,
    /// 系统忙，视频质量降低
    XYVideoState_LowAsLocalHardware,
    /// 对方网络不稳定
    XYVideoState_LowAsRemote,
    /// 网络不稳定，请稍候
    XYVideoState_NetworkError,
    /// WiFi信号不稳定
    XYVideoState_LocalWifiIssue,
};

/// 视频流状态
typedef NS_ENUM(NSUInteger, XYVideoStreamState) {
    /// 闲置
    XYVideoStreamState_Idle,
    /// 监听
    XYVideoStreamState_Observing,
    /// 视频关闭
    XYVideoStreamState_Mute,
    /// 请求中
    XYVideoStreamState_Requesting,
    /// 带宽不足
    XYVideoStreamState_NoBandwidth,
    /// 无解码器
    XYVideoStreamState_NoDecoder,
    /// 正常接收
    XYVideoStreamState_Received,
    /// 电话
    XYVideoStreamState_Telephone,
    /// 只有音频
    XYVideoStreamState_AudioOnly,
    /// 只收到音频
    XYVideoStreamState_ReceivedAudioOnly,
    /// 巡检
    XYVideoStreamState_DeviceDetection,
    /// cpu能力不足
    XYVideoStreamState_NoCpuResource,
};

/// 音频或视频关闭原因
typedef NS_ENUM(NSUInteger, XYMuteReason) {
    /// 用户关闭视频
    XYMuteReason_User,
    /// 带宽限制
    XYMuteReason_BWLimit,
    /// 没有输入
    XYMuteReason_NoInput,
    /// 会控关闭
    XYMuteReason_ConfMgmt,
    /// 硬件检测关闭
    XYMuteReason_DeviceDetection,
    /// 电话关闭
    XYMuteReason_PhoneCall,
};

/// 分享content类型
typedef NS_ENUM(NSUInteger, XYContentType) {
    /// 分享图片
    XYContentType_Image,
    /// 分享屏幕
    XYContentType_Screen,
};

/// 共享状态
typedef NS_ENUM(NSUInteger, XYContentState) {
    /// 闲置，停止发送
    XYContentState_Idle,
    /// 开始发送
    XYContentState_Acquiring,
    /// 发送中
    XYContentState_Sending,
    /// 不支持
    XYContentState_Unsupported,
    /// 带宽不足
    XYContentState_NoBandwidth,
    /// 未知
    XYContentState_Unknown,
};

/// 白板状态
typedef NS_ENUM(NSUInteger, XYWhiteboardState) {
    /// 未知
    XYWhiteboardState_Unknown,
    /// 开启
    XYWhiteboardState_Start,
    /// 关闭
    XYWhiteboardState_Stop,
    /// 错误
    XYWhiteboardState_Error,
};

/// 白板操作
typedef NS_ENUM(NSUInteger, XYWhiteboardOperation) {
    /// 设置画布
    XYWhiteboardOperation_setCanvas,
    /// 画线
    XYWhiteboardOperation_Draw,
    /// 清理
    XYWhiteboardOperation_Clear,
    /// 所有线条
    XYWhiteboardOperation_All,
    /// 撤销
    XYWhiteboardOperation_Undo,
    /// 重做
    XYWhiteboardOperation_Redo,
    /// 图形
    XYWhiteboardOperation_Graph,
    /// 擦除一整条线
    XYWhiteboardOperation_LineErase,
    /// 编辑
    XYWhiteboardOperation_Edit,
    /// 未知
    XYWhiteboardOperation_Unkown,
};

/// 标注状态
typedef NS_ENUM(NSUInteger, XYAnnotationStatus) {
    /// 未知
    XYAnnotationStatus_Unknown,
    /// 开启
    XYAnnotationStatus_Start,
    /// 关闭
    XYAnnotationStatus_Stop,
    /// 错误
    XYAnnotationStatus_Error,
};

/// 画线类型
typedef NS_ENUM(NSUInteger, XYAnnotationOperation) {
    /// 画线
    XYAnnotationOperation_DrawLine,
    /// 清理
    XYAnnotationOperation_Clear,
    /// 获取
    XYAnnotationOperation_GetAllLine,
    /// 撤销
    XYAnnotationOperation_Undo,
    /// 重做
    XYAnnotationOperation_Redo,
    /// 获取状态
    XYAnnotationOperation_Status,
    /// 获取信息
    XYAnnotationOperation_Info,
    /// 画线反馈
    XYAnnotationOperation_Feedback,
    /// 无操作
    XYAnnotationOperation_None,
};

/// 会控控制麦克风状态
typedef NS_ENUM(NSUInteger, XYMeetingCtrlMicState) {
    /// 无操作
    XYMeetingCtrlMicState_Default,
    /// 静音
    XYMeetingCtrlMicState_Mute,
    /// 强制静音
    XYMeetingCtrlMicState_MuteMandatory,
    /// 取消静音
    XYMeetingCtrlMicState_Unmute,
    /// 暂时取消静音
    XYMeetingCtrlMicState_UnmuteTemporary,
};

/// 字幕字体大小
typedef NS_ENUM(NSUInteger, XYCaptionFontSize) {
    /// 小
    XYCaptionFontSize_Small,
    /// 中
    XYCaptionFontSize_Middle,
    /// 大
    XYCaptionFontSize_Big,
};

/// 字幕显示位置
typedef NS_ENUM(NSUInteger, XYCaptionLocation) {
    /// 上部
    XYCaptionLocation_Top,
    /// 中部
    XYCaptionLocation_Middle,
    /// 底部
    XYCaptionLocation_Bottom,
};

/// 录制状态
typedef NS_ENUM(NSUInteger, XYRecordingState) {
    /// 录制开始
    XYRecordingState_Start,
    /// 录制暂停
    XYRecordingState_Pause,
    /// 录制继续
    XYRecordingState_Active,
    /// 录制结束
    XYRecordingState_Stop,
};

/// 虚拟背景配置
typedef NS_ENUM(NSInteger, XYVirtualBackgroundMode) {
    /// 无背景
    XYVirtualBackgroundModeClose,
    /// 背景模糊
    XYVirtualBackgroundModeBlur,
    /// 图片选择
    XYVirtualBackgroundModeImage,
};

/// 美颜
typedef NS_ENUM(NSUInteger, XYBeautyEffectType) {
    /// 无美颜
    XYBeautyEffectType_None,
    /// 美白
    XYBeautyEffectType_Whitening,
    /// 磨皮
    XYBeautyEffectType_Buffing,
    /// 红润
    XYBeautyEffectType_Blush,
};

/// 滤镜
typedef NS_ENUM(NSUInteger, XYVideoFilterType) {
    /// 无滤镜
    XYVideoFilterType_None,
    /// 简洁
    XYVideoFilterType_Concise,
    /// 自然
    XYVideoFilterType_Natural,
    /// 精致
    XYVideoFilterType_Exquisite,
    /// 宁静
    XYVideoFilterType_Peaceful,
    /// 柔和
    XYVideoFilterType_Soft,
    /// 轻奢
    XYVideoFilterType_LightLuxury,
    /// 青春
    XYVideoFilterType_Youth,
    /// 雨后
    XYVideoFilterType_Rain,
    /// 校园
    XYVideoFilterType_Campus,
    /// 热情
    XYVideoFilterType_Warmth,
    /// 清新
    XYVideoFilterType_Refreshing,
    /// 银河
    XYVideoFilterType_Galaxy,
    /// 清澈
    XYVideoFilterType_Pure,
    /// 清晨
    XYVideoFilterType_Dawn,
    /// 梦幻
    XYVideoFilterType_Dream,
    /// 高光
    XYVideoFilterType_HighLight,
    /// 典雅
    XYVideoFilterType_Elegant,
    /// 怀旧
    XYVideoFilterType_Nostalgia,
    /// 暗调
    XYVideoFilterType_Dark,
    /// 老电影
    XYVideoFilterType_Retro,
};

/// 网络测试结果
typedef NS_ENUM(NSInteger, XYNetworkTestCode) {
    /// 正常
    XYNetworkTestCode_NoError,
    /// 终止
    XYNetworkTestCode_Expired,
    /// 强制结束
    XYNetworkTestCode_Killed,
    /// tcp连接失败
    XYNetworkTestCode_CreateTCPSocketFail,
    /// udp连接失败
    XYNetworkTestCode_CreateUDPSocketFail,
    /// 无网络连接
    XYNetworkTestCode_LostConnection,
    /// 未找到服务器
    XYNetworkTestCode_NotFoundServer,
};

/// 会控互动状态
typedef NS_ENUM(NSUInteger, XYInteractiveState) {
    /// 开始
    XYInteractiveState_Start,
    /// 公布结果
    XYInteractiveState_Announce,
    /// 关闭
    XYInteractiveState_Close,
    /// 结束
    XYInteractiveState_End,
};

/// FECC操作指令
typedef NS_ENUM(NSUInteger, XYFECCCommand) {
    XYFECCCommand_TurnLeft,
    XYFECCCommand_StepLeft,
    XYFECCCommand_TurnRight,
    XYFECCCommand_StepRight,
    XYFECCCommand_TurnStop,
    XYFECCCommand_TILT_Camera_TurnUp,
    XYFECCCommand_TILT_Camera_StepUp,
    XYFECCCommand_TILT_Camera_TurnDown,
    XYFECCCommand_TILT_Camera_StepDown,
    XYFECCCommand_TILT_Camera_TurnStop,
    XYFECCCommand_ZoomIn,
    XYFECCCommand_ZoomOut,
    XYFECCCommand_StepZoomIn,
    XYFECCCommand_StepZoomOut,
    XYFECCCommand_ZoomStop,
    XYFECCCommand_Move_TurnLeft,
    XYFECCCommand_Move_StepLeft,
    XYFECCCommand_Move_TurnRight,
    XYFECCCommand_Move_StepRight,
    XYFECCCommand_Move_TurnUp,
    XYFECCCommand_Move_StepUp,
    XYFECCCommand_Move_TurnDown,
    XYFECCCommand_Move_StepDown,
    XYFECCCommand_Move_Stop,
    XYFECCCommand_Unknown,
};
#endif /* XYRTCEngineEnums_h */
